#include "wiced.h"
#include "../simpleCli.h"
#ifdef USE_SUSI_API_TEST
#include "../susiTest/susiGpioTest.h"
#include "../susiTest/susiI2CTest.h"
#include "../susiTest/susiSPITest.h"
#include "../susiTest/susiADCTest.h"
#else
#include "GpioTest.h"
#include "I2CTest.h"
#include "SPITest.h"
#include "ADCTest.h"
#endif
#include "UartTest.h"
#include "WiFiTest.h"
#include "BTTest.h"

#include "wiced_bt.h"
#include "wiced_bt_smartbridge.h"
#include "wiced_bt_types.h"
#include "wiced_bt_ble.h"

#define BLE_Scan_TIMEOUT		100;

extern void smartbridge_scan_result_callback( wiced_bt_ble_scan_results_t *p_scan_result, uint8_t *p_adv_data );
extern wiced_bt_smart_scan_complete_callback_t      app_scan_complete_callback;

static const wiced_bt_device_address_t bt_smartbridge_custom_address    = {0x11, 0x22, 0x34, 0x45, 0x56, 0x67};
static const wiced_bt_device_address_t bt_smartbridge_address_mask      = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
static wiced_worker_thread_t          connect_worker_thread;
static wiced_result_t scan_complete_handler  ( void );
static wiced_result_t Find_Dev( char *Name );

static wiced_bool_t bInit=WICED_FALSE;
static wiced_bool_t bComp;
static char DestName[20]="WISE-1530_BLE";


/* Scan settings */
static const wiced_bt_smart_scan_settings_t scan_settings =
{
    .type              = BT_SMART_ACTIVE_SCAN,
    .filter_policy     = FILTER_POLICY_NONE,
    .filter_duplicates = DUPLICATES_FILTER_ENABLED,
    .interval          = 96,
    .window            = 48,
    .duration_second   = 3,
};

/* SmartBridge security settings */
static const wiced_bt_smart_security_settings_t security_settings =
{
    .timeout_second              = 15,
    .io_capabilities             = BT_SMART_IO_KEYBOARD_DISPLAY,
    .authentication_requirements = BT_SMART_AUTH_REQ_BONDING_AND_PASSKEY_ENTRY,
    .oob_authentication          = BT_SMART_OOB_AUTH_NONE,
    .max_encryption_key_size     = 16,
    .master_key_distribution     = BT_SMART_DISTRIBUTE_ALL_KEYS,
    .slave_key_distribution      = BT_SMART_DISTRIBUTE_ALL_KEYS,
};


static int InputName(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0){
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (Name:%s)",DestName);
		return 0;
	}

	int leng;
	int result;
	int temp[4];

	console_printf("\r\n*) Please input destination Name: ");
	GetStr_wait(DestName, &leng);

	console_printf("\r\nDest. Name=%s", DestName);

	return 0;
}

static int BLETest(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	int timeout=BLE_Scan_TIMEOUT;

	BLE_init();
	bComp=WICED_FALSE;
    smartbridge_helper_delete_scan_result_list();
    wiced_bt_ble_scan( BTM_BLE_SCAN_TYPE_HIGH_DUTY, WICED_TRUE, smartbridge_scan_result_callback );
    console_printf("Sacn BLE device...\r\n");

    while(bComp==WICED_FALSE){
    	wiced_rtos_delay_milliseconds( 200 );
    	timeout--;
    	if(timeout==0){
    		console_printf( ("BLE test timeout ===> NG\r\n") );
    		getchar();
    		return true;
    	}
    }
    if(Find_Dev((void *)DestName)!=WICED_SUCCESS)
    	console_printf("Not find %s device ===> NG\r\n", DestName);
    else
    	console_printf("Find %s device ===> OK\r\n", DestName);

	return true;
}

int BTTest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	simple_cmds cmds[] = {
            { "Bluetooth list" /*title*/,           NULL, NULL  },
            { "BLE Scan",                           0, BLETest  },
			{ "Destination Name",                  	0, InputName},
            { NULL,                                 NULL, NULL  }
    };

	simple_cmds cmds_pre[] = {
			{ "I/O Function Test" /*title*/,          		0, NULL          },
			{ "I2C",										0, I2CTest_Menu  },
			{ "SPI0",										0, SPITest_Menu  },
			{ "GPIO Test",									0, GpioTest_Menu  },
			{ "Uart-2 Test",								0, UartTest_Menu },
			{ "ADC",										0, ADCTest_Menu  },
			{ "Wi-Fi",										0, WiFiTest_Menu },
			{ "Bluetooth",									0, BTTest_Menu   },

			{ NULL,											0, NULL          }
	};


	if(	WiFi_Conn()==(int)WICED_SUCCESS)
		return simple_cli_list(cmds);
	else
		return simple_cli_list(cmds_pre);
}

int BLE_Test(void)
{
	int timeout=BLE_Scan_TIMEOUT;


	if(BLE_init()!=WICED_SUCCESS){
		return -1;
	}

    bComp=WICED_FALSE;

    smartbridge_helper_delete_scan_result_list();
    wiced_bt_ble_scan( BTM_BLE_SCAN_TYPE_HIGH_DUTY, WICED_TRUE, smartbridge_scan_result_callback );

    while(bComp==WICED_FALSE){
    	wiced_rtos_delay_milliseconds( 200 );
    	timeout--;
    	if(timeout==0){
    		console_printf( ("BLE test timeout ===> ") );
    		return -1;
    	}
    }

    if(Find_Dev((void *)BLE_TARGET_NAME)!=WICED_SUCCESS){
    	console_printf("Not find %s device ===> ", BLE_TARGET_NAME);
    	return -1;
    }

    return WICED_SUCCESS;
}


int BLE_init(void)
{
	wiced_result_t result;

	if(bInit==WICED_TRUE){
		return WICED_SUCCESS;
	}

	WPRINT_APP_DEBUG( ("BT init...\r\n") );
	wiced_bt_init_address( (const wiced_bt_device_address_t*)&bt_smartbridge_custom_address, (const wiced_bt_device_address_t*)&bt_smartbridge_address_mask);

	/* Initialise WICED Bluetooth Framework */
	result=wiced_bt_init( WICED_BT_HCI_MODE, "SmartBridge Device" );
	if(result!=WICED_BT_SUCCESS){
		console_printf( ("NG-1 ===> ") );
		return -1;
	}
    /* Initialise WICED SmartBridge */
	result=wiced_bt_smartbridge_init();
	if(result!=WICED_BT_SUCCESS){
		console_printf( ("NG-2 ===> ") );
		return -1;
	}

	result=wiced_network_up( WICED_STA_INTERFACE, WICED_USE_EXTERNAL_DHCP_SERVER, NULL );
	if(result!=WICED_SUCCESS){
		console_printf( ("NG-3 ===> ") );
		return -1;
	}

	app_scan_complete_callback=scan_complete_handler;
	bInit=WICED_TRUE;

    return WICED_SUCCESS;
}

/* Scan complete handler. Scan complete event reported via this callback.
 * It runs on the WICED_NETWORKING_WORKER_THREAD context.
 */
static wiced_result_t scan_complete_handler( void )
{
    WPRINT_APP_DEBUG( ("[App] Scan complete\r\n") );

    bComp=WICED_TRUE;
    /* Scan duration is complete */
    return WICED_SUCCESS;
}

static wiced_result_t Find_Dev( char *Name )
{
    uint32_t count = 0;
    wiced_bt_smart_scan_result_t* temp = NULL;
    /* List Scan Result */
    wiced_bt_smartbridge_get_scan_result_list( &temp, &count );

    while ( temp != NULL ){
    	if(temp->filter_display == WICED_FALSE)
    		WPRINT_APP_DEBUG( ("BLE name:%s\r\n", temp->remote_device.name) );
    	if(strcmp(temp->remote_device.name, Name)==0){
    		return WICED_SUCCESS;
    	}

    	temp = temp->next;
    }

    return -1;
}
