#include "wiced.h"
#include "../simpleCli.h"
#include "GpioTest.h"

#define LED0	( WICED_GPIO_6 )
#define LED1	( WICED_GPIO_16 )


static int GPIO_Test(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;
		switch(index){
				case 0:
					wiced_gpio_output_high(LED0);//LED0=1;
					break;

				case 1:
					wiced_gpio_output_low(LED0);//LED0=0;
					break;

				case 2:
					wiced_gpio_output_high(LED1);//LED1=1;
					break;

				case 3:
					wiced_gpio_output_low(LED1);//LED1=0;
					break;

				default:
					wiced_gpio_output_low(LED0);//LED0=0;
					wiced_gpio_output_low(LED1);//LED1=0;
					break;
	}

	return 0;
}

static void publish_callback( void* arg )
{
    printf ( "Push wake up button.\r\n");
    wiced_rtos_delay_microseconds(5000);
}

static int GoStandby(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	HAL_PWR_DisableWakeUpPin(PWR_WakeUp_Pin2);
	PWR_ClearFlag(PWR_FLAG_WU);
	HAL_PWR_EnableWakeUpPin(PWR_WakeUp_Pin2);
	console_printf("Enter standby mode, push wake up button to wake up.\r\n");
	wiced_rtos_delay_microseconds(300);
	PWR_EnterSTANDBYMode();
	return 0;
}


static int GoSleep(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	wiced_gpio_input_irq_enable( WICED_BUTTON_WKUP, IRQ_TRIGGER_RISING_EDGE, publish_callback, NULL);
	console_printf("Enter sleep mode, press any key to wake up.\r\n");
	wiced_rtos_delay_microseconds(300);

	SysTick->CTRL &= ~SysTick_CTRL_TICKINT_Msk;
	PWR_EnterSleepMode(PWR_MainRegulator_ON, PWR_SLEEPEntry_WFI);
	SysTick->CTRL |= SysTick_CTRL_TICKINT_Msk;

	return 0;
}

int GpioTest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	simple_cmds cmds[] = {
				{ "GPIO Test" /*title*/,  					0,NULL },
				{ "GPIO-0 High",							0,GPIO_Test },
				{ "GPIO-0 Low",								1,GPIO_Test },
				{ "GPIO-1 High",							2,GPIO_Test },
				{ "GPIO-1 Low",								3,GPIO_Test },
				{ "Standby mode",     						0,GoStandby},
				{ "Sleep mode", 							0,GoSleep},
				{ NULL,										0,NULL }
		};
	return simple_cli_list(cmds);
}

int Gpio_Auto_Test(void)
{
	wiced_bool_t bLed1;

	wiced_gpio_output_high( WICED_LED2 );
	bLed1=wiced_gpio_input_get(WICED_LED1);
	if(bLed1!=WICED_TRUE){
		console_printf("GPIO_err1 ===> ");
		return -1;
	}

	wiced_gpio_output_low( WICED_LED2 );
	bLed1=wiced_gpio_input_get(WICED_LED1);
	if(bLed1!=WICED_FALSE){
		console_printf("GPIO_err2 ===> ");
		return -1;
	}

	return WICED_SUCCESS;
}
