#include "wiced.h"
#include "Susi4.h"
#include "susi_pwm.h"


static Susi_PWM_Para_t  pwmHandle[PWM_MAX_BUS];
static bool     g_bInited = false;

void SusiPWM_Init(void)
{
	if(!g_bInited) {
		int i;

		for(i=0;i<PWM_MAX_BUS;i++)
		{
			pwmHandle[i].ch = i;
            pwmHandle[i].period = 1000;
            pwmHandle[i].duty = 10;
			wiced_pwm_init(pwmHandle[i].ch, pwmHandle[i].period, pwmHandle[i].duty);
            wiced_pwm_start(pwmHandle[i].ch);
		}
		g_bInited = true;
	}
	return;
}

SusiStatus_t SUSI_API SusiPWMGetPeriod(
	SusiId_t Id,		/* IN	PWM Bus Id */
	uint32_t *pPeriod	/* OUT	period */
    )
{
    if(Id >= PWM_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiPWM_Init();

    *pPeriod = pwmHandle[Id].period;

	return SUSI_STATUS_SUCCESS;
}

SusiStatus_t SUSI_API SusiPWMSetPeriod(
	SusiId_t Id,		/* IN	PWM Bus Id */
	uint32_t period		/* IN	period*/
    )
{
    wiced_result_t result;
    
    if(Id >= PWM_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiPWM_Init();

    pwmHandle[Id].period = period;
    result=wiced_pwm_init(pwmHandle[Id].ch, pwmHandle[Id].period, pwmHandle[Id].duty);
    if(result != WICED_SUCCESS)
        return result;

    result=wiced_pwm_start(pwmHandle[Id].ch);
    if(result != WICED_SUCCESS)
        return result;
    
    return SUSI_STATUS_SUCCESS;
}

SusiStatus_t SUSI_API SusiPWMSetDutyCycle(
	SusiId_t Id,			/* IN	PWM Bus Id */
	uint32_t dutyCycle		/* IN	duty cycle(0%~100%)*/
    )
{
    wiced_result_t result;
    
    if(Id >= PWM_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiPWM_Init();

    pwmHandle[Id].duty= dutyCycle;
    result=wiced_pwm_init(pwmHandle[Id].ch, pwmHandle[Id].period, pwmHandle[Id].duty);
    if(result != WICED_SUCCESS)
        return result;

    result=wiced_pwm_start(pwmHandle[Id].ch);
    if(result != WICED_SUCCESS)
        return result;

    return SUSI_STATUS_SUCCESS;
}

