#include "wiced.h"
#include "../simpleCli.h"
#include "Susi4.h"
#include "susiGpioTest.h"
#include "susi_gpio.h"


#define LED0	( WICED_LED_INDEX_1 )
#define LED1	( WICED_LED_INDEX_2 )

static int GPIO_Test(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;
		switch(index){
				case 0:
					SusiGPIOSetLevel(LED0, 0, SUSI_GPIO_HIGH);//LED0=1;
					console_printf("Susi GPIO-0=1\r\n");
					break;

				case 1:
				    SusiGPIOSetLevel(LED0, 0, SUSI_GPIO_LOW);//LED0=0;
				    console_printf("Susi GPIO-0=0\r\n");
					break;

				case 2:
				    SusiGPIOSetLevel(LED1, 0, SUSI_GPIO_HIGH);//LED1=1;
				    console_printf("Susi GPIO-1=1\r\n");
					break;

				case 3:
				    SusiGPIOSetLevel(LED1, 0, SUSI_GPIO_LOW);//LED1=0;
				    console_printf("Susi GPIO-1=0\r\n");
					break;

				default:
				    SusiGPIOSetLevel(LED0, 0, SUSI_GPIO_LOW);//LED0=0;
					SusiGPIOSetLevel(LED1, 0, SUSI_GPIO_LOW);//LED1=0;
					break;
	}

	return 0;
}

static void publish_callback( void* arg )
{
    printf ( "Push wake up button.\r\n");
    wiced_rtos_delay_microseconds(5000);
}

static int GoStandby(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	HAL_PWR_DisableWakeUpPin(PWR_WakeUp_Pin2);
	PWR_ClearFlag(PWR_FLAG_WU);
	HAL_PWR_EnableWakeUpPin(PWR_WakeUp_Pin2);
	console_printf("Enter standby mode, push wake up button to wake up.\r\n");
	wiced_rtos_delay_microseconds(300);
	PWR_EnterSTANDBYMode();
	return 0;
}


static int GoSleep(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	wiced_gpio_input_irq_enable( WICED_BUTTON_WKUP, IRQ_TRIGGER_RISING_EDGE, publish_callback, NULL);
	console_printf("Enter sleep mode, press any key to wake up.\r\n");
	wiced_rtos_delay_microseconds(300);

	SysTick->CTRL &= ~SysTick_CTRL_TICKINT_Msk;
	PWR_EnterSleepMode(PWR_MainRegulator_ON, PWR_SLEEPEntry_WFI);
	SysTick->CTRL |= SysTick_CTRL_TICKINT_Msk;

	return 0;
}

int GpioTest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	simple_cmds cmds[] = {
				{ "GPIO Test" /*title*/,  					0,NULL },
				{ "GPIO-0 High",							0,GPIO_Test },
				{ "GPIO-0 Low",								1,GPIO_Test },
				{ "GPIO-1 High",							2,GPIO_Test },
				{ "GPIO-1 Low",								3,GPIO_Test },
				{ "Standby mode",     						0,GoStandby},
				{ "Sleep mode", 							0,GoSleep},
				{ NULL,										0,NULL }
		};
	return simple_cli_list(cmds);
}

int Gpio_Auto_Test(void)
{
    uint32_t bLed;

	SusiGPIOSetLevel(LED1, 0, SUSI_GPIO_HIGH);
	SusiGPIOGetLevel(LED0, 0, &bLed);
	if(bLed!=SUSI_GPIO_HIGH){
		console_printf("GPIO_err1 ===> ");
		return -1;
	}

	SusiGPIOSetLevel(LED1, 0, SUSI_GPIO_LOW);
	SusiGPIOGetLevel(LED0, 0, &bLed);;
	if(bLed!=SUSI_GPIO_LOW){
		console_printf("GPIO_err2 ===> ");
		return -1;
	}

	return WICED_SUCCESS;
}
