#include "wiced.h"
#include "../simpleCli.h"
#include "Susi4.h"
#include "susiSPITest.h"
#include "susi_spi.h"

//HDC1050 Slave
#define N25Q032A_CMD_READ_ID        0x9F// 0x9E/0x9F
#define N25Q032A_ID					0x20BA1610
//HDC1050 Offset
#define N25Q032A_MANUFACTURER_ID    0x20//	Byte 1 Manufacturer ID

#define SPI_ID                      0

static int gBitRate= 0;
static unsigned char gOffsetAddress = N25Q032A_CMD_READ_ID;


static int BitRate(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len, gBitRate == 0 ? " (1MHz)" : " (100KHz)");
		return 0;
	}
	console_printf("\r\n*) Please input the bitrate ( 0: 1MHz , 1: 100KHz ): \r\n");
	gBitRate = GetInt_wait();
    
	if(gBitRate==1){
	    SusiSPISetFrequency(SPI_ID, 100);
	}
	else{
	    SusiSPISetFrequency(SPI_ID, 1000);
	}
	return 0;
}

static int OffsetAddress(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (0x%02X)",gOffsetAddress);
		return 0;
	}
	console_printf("\r\n*) Please input the offset address ( 0x00 ~ 0xFF ): \r\n");
	gOffsetAddress = (char)(GetInt_wait()&0xFF);
    
    if(gOffsetAddress < 0 || gOffsetAddress > 0xFF)
        gOffsetAddress = N25Q032A_CMD_READ_ID;
	return 0;
}

static int Read(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	SusiStatus_t result;
	unsigned char tx[5]={N25Q032A_CMD_READ_ID, 0, 0, 0, 0};
	unsigned char rx[5] = {0, 0, 0, 0, 0};

    SusiSPIReadWrite(SPI_ID, tx, rx, sizeof(tx));
    if(result == SUSI_STATUS_SUCCESS){
        console_printf("\r\n");
        console_printf("Susi SPI Result: 0x%2X 0x%2X 0x%2X 0x%2X\n",rx[1],rx[2],rx[3],rx[4]);
        console_printf("\r\n");
    }
    else{
        console_printf("SPI Read error.\r\n");
    }

	return true;
}

int SPITest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;
    
	simple_cmds cmds[] = {
            { "SPI Test" /*title*/,  					0,NULL },
            { "BitRate",							    index,BitRate },
            { "Offset Address",							index,OffsetAddress },
            { "Read",			                        index,Read },
            { NULL,										0,NULL }
    };

	return simple_cli_list(cmds);
}

int SPI_Auto_Test(void)
{
    SusiStatus_t result;
	unsigned char tx[5]={N25Q032A_CMD_READ_ID, 0, 0, 0, 0};
	unsigned char rx[5] = {0, 0, 0, 0, 0};

	result=SusiSPISetFrequency(SPI_ID, 1000);
    if(result != SUSI_STATUS_SUCCESS){
        console_printf("SPI_err1 ===> ");
        return result;
    }

	result=SusiSPIReadWrite(SPI_ID, tx, rx, sizeof(tx));
    if(result != SUSI_STATUS_SUCCESS){
    	console_printf("SPI_err2 ===> ");
    	return result;
    }

    if((rx[1]<<24 | rx[2]<<16 | rx[3]<<8 | rx[4])==N25Q032A_ID)
    	return WICED_SUCCESS;
    else{
    	console_printf("N25Q032A_ID=%d ===>", (rx[1]<<24 | rx[2]<<16 | rx[3]<<8 | rx[4]));
    	return -1;
    }
}
