#include "wiced_platform.h"
#include "hdc1050.h"

static wiced_i2c_device_t  _i2c_dev;

void HDC1050_Init(void)
{
	wiced_result_t result;
    _i2c_dev.port           = WICED_I2C_1;
    _i2c_dev.address        = TI_HDC1050_DEVICE_ADDR;
    _i2c_dev.address_width  = I2C_ADDRESS_WIDTH_7BIT;
    _i2c_dev.flags          = I2C_DEVICE_NO_DMA;
    _i2c_dev.speed_mode     = I2C_STANDARD_SPEED_MODE;
    result=wiced_i2c_init(&_i2c_dev);

    if(result==WICED_SUCCESS)
    	WPRINT_APP_INFO(("wiced_i2c_init OK.\r\n"));
    else
    	WPRINT_APP_INFO(("wiced_i2c_init Fail.\r\n"));
}	

void HDC1050_UnInit(void)
{
    wiced_i2c_deinit(&_i2c_dev);	
}

uint16_t  HDC1050_GetVendorID(void)
{
	uint16_t   RetVal = 0;
#if 0
	int32_t    i;
	uint8_t    txBuff[10];
	uint8_t    rxBuff[10];


	txBuff[0] = TI_HDC1050_MANUFACTURER_ID_ADDR;
    i2cTransaction.writeBuf = txBuff;
    i2cTransaction.writeCount = 1;
    i2cTransaction.readBuf = rxBuff;
    i2cTransaction.readCount = 2;
    if(i2cHandle>0)
    	i = I2C_transfer(i2cHandle, &i2cTransaction);
    
    if(i)
    {	
	 	Debug("[Vendor ID]: 0x%02x 0x%02x \r\n",rxBuff[0],rxBuff[1]);
		RetVal  = ((uint16_t)rxBuff[0]) << 8 | (uint16_t)	rxBuff[1];
	}	
#endif	 	
	return RetVal; 	
}	

void  HDC1050_GetSensorData(float *Temperature, uint16_t *Humidity)
{
	float   _temp=0;
	uint16_t _hmdty=0;

    wiced_i2c_message_t msg;
	uint8_t reg=TI_HDC1050_TEMPERATURE_ADDR;
	uint8_t rxBuff[4]={0};
	wiced_result_t result;

	memset(rxBuff, 0, sizeof(rxBuff));
	result=wiced_i2c_init_tx_message(&msg, &reg, sizeof(reg), 3, I2C_DEVICE_NO_DMA);

    if(result != WICED_SUCCESS){
    	WPRINT_APP_INFO(("I2C_err2 ===> "));
       	return;
    }
    result = wiced_i2c_transfer(&_i2c_dev, &msg, 1);
    wiced_rtos_delay_milliseconds(50);
    
    result=wiced_i2c_init_rx_message(&msg, rxBuff, sizeof(rxBuff), 3, I2C_DEVICE_NO_DMA);
    if(result != WICED_SUCCESS){
    	WPRINT_APP_INFO(("I2C_err3 ===> "));
       	return;
    }
    result = wiced_i2c_transfer(&_i2c_dev, &msg, 1);

    if(result==WICED_SUCCESS)
	{
    	WPRINT_APP_INFO(("HDC1050_GetSensorData OK.\r\n"));

		_temp =(float)(( rxBuff[0] << 8) | (rxBuff[1] ));
		_temp = (_temp*165)/65536-40;
		_hmdty = (rxBuff[2] << 8) | (rxBuff[3] );
		_hmdty = (_hmdty*100)/65536;

	}
    else
    	WPRINT_APP_INFO(("HDC1050_GetSensorData Fail.\r\n"));

	if(Temperature!=NULL)
		*Temperature = _temp;
	if(Humidity!=NULL)
		*Humidity = _hmdty;
}

#if defined(BOARD_TEST)
void HDC1050_Test(void)
{
	wiced_i2c_message_t msg[2];
	uint8_t reg=TI_HDC1050_MANUFACTURER_ID_ADDR;
	uint8_t val[5]={0};
	wiced_result_t result;

    if(result == WICED_SUCCESS)
    	result=wiced_i2c_init_tx_message(&msg[0], &reg, sizeof(reg), 3, I2C_DEVICE_NO_DMA);

    /* Initialize RX message. */
    if(result == WICED_SUCCESS)
    	result=wiced_i2c_init_rx_message(&msg[1], val, sizeof(val), 3, I2C_DEVICE_NO_DMA);


    /* Transfer. */
    if(result == WICED_SUCCESS)
        result = wiced_i2c_transfer(&_i2c_dev, msg, ARRAYSIZE(msg));
    wiced_rtos_delay_microseconds(100);

    if(result == WICED_SUCCESS){
    	WPRINT_APP_INFO(("Read from 0x%02X--0x%02X\r\n",TI_HDC1050_DEVICE_ADDR, TI_HDC1050_MANUFACTURER_ID_ADDR));
    	WPRINT_APP_INFO(("Result: 0x%02X%02X\r\n",val[0],val[1]));
    	WPRINT_APP_INFO(("\r\n"));
    }
    else{
    	//result=wiced_i2c_deinit(&_i2c_dev);
    	WPRINT_APP_INFO(("Read_HDC105_ID error.\r\n"));
    }

    //wiced_i2c_deinit(&_i2c_dev);
    wiced_rtos_delay_microseconds(30);
}
#endif


